@@ -26,7 +26,6 @@ import Data.Aeson (object, (.=))
 import           Data.List                          (foldl')
 import           Data.List                          (nub)
 import qualified Data.Map                           as Map
-import           Data.Maybe                         (fromMaybe)
 import           Data.Monoid
 import           Data.Text                          (Text)
 import qualified Data.Text                          as T
@@ -207,17 +206,14 @@ class RenderRoute site => Yesod site where
     -- method return that already created value. That way, you can use that
     -- same @Logger@ for printing messages during app initialization.
     --
-    -- Default: Sends to stdout and automatically flushes on each write.
+    -- Default: the 'defaultMakeLogger' function.
     makeLogger :: site -> IO Logger
-    makeLogger _ = do
-        loggerSet' <- newStdoutLoggerSet defaultBufSize
-        (getter, _) <- clockDateCacher
-        return $! Logger loggerSet' getter
+    makeLogger _ = defaultMakeLogger
 
     -- | Send a message to the @Logger@ provided by @getLogger@.
     --
-    -- Default implementation: checks if the message should be logged using
-    -- 'shouldLog' and, if so, formats using 'formatLogMessage'.
+    -- Default: the 'defaultMessageLoggerSource' function, using
+    -- 'shouldLogIO' to check whether we should log.
     messageLoggerSource :: site
                         -> Logger
                         -> Loc -- ^ position in source code
@@ -225,10 +221,7 @@ class RenderRoute site => Yesod site where
                         -> LogLevel
                         -> LogStr -- ^ message
                         -> IO ()
-    messageLoggerSource a logger loc source level msg = do
-        sl <- shouldLogIO a source level
-        when sl $
-            formatLogMessage (loggerDate logger) loc source level msg >>= loggerPutStr logger
+    messageLoggerSource site = defaultMessageLoggerSource $ shouldLogIO site
 
     -- | Where to Load sripts from. We recommend the default value,
     -- 'BottomOfBody'.  Alternatively use the built in async yepnope loader:
@@ -260,9 +253,9 @@ class RenderRoute site => Yesod site where
 
     -- | Should we log the given log source/level combination.
     --
-    -- Default: Logs everything at or above 'logLevel'
+    -- Default: the 'defaultShouldLog' function.
     shouldLog :: site -> LogSource -> LogLevel -> Bool
-    shouldLog _ _ level = level >= LevelInfo
+    shouldLog _ = defaultShouldLog
 
     -- | Should we log the given log source/level combination.
     --
@@ -300,6 +293,51 @@ class RenderRoute site => Yesod site where
     yesodWithInternalState _ _ = bracket createInternalState closeInternalState
     {-# INLINE yesodWithInternalState #-}
 
+-- | Default implementation of 'makeLogger'. Sends to stdout and
+-- automatically flushes on each write.
+--
+-- Since 1.4.10
+defaultMakeLogger :: IO Logger
+defaultMakeLogger = do
+    loggerSet' <- newStdoutLoggerSet defaultBufSize
+    (getter, _) <- clockDateCacher
+    return $! Logger loggerSet' getter
+
+-- | Default implementation of 'messageLoggerSource'. Checks if the
+-- message should be logged using the provided function, and if so,
+-- formats using 'formatLogMessage'. You can use 'defaultShouldLogIO'
+-- as the provided function.
+--
+-- Since 1.4.10
+defaultMessageLoggerSource ::
+       (LogSource -> LogLevel -> IO Bool) -- ^ Check whether we should
+                                          -- log this
+    -> Logger
+    -> Loc -- ^ position in source code
+    -> LogSource
+    -> LogLevel
+    -> LogStr -- ^ message
+    -> IO ()
+defaultMessageLoggerSource ckLoggable logger loc source level msg = do
+    loggable <- ckLoggable source level
+    when loggable $
+        formatLogMessage (loggerDate logger) loc source level msg >>=
+        loggerPutStr logger
+
+-- | Default implementation of 'shouldLog'. Logs everything at or
+-- above 'LevelInfo'.
+--
+-- Since 1.4.10
+defaultShouldLog :: LogSource -> LogLevel -> Bool
+defaultShouldLog _ level = level >= LevelInfo
+
+-- | A default implementation of 'shouldLogIO' that can be used with
+-- 'defaultMessageLoggerSource'. Just uses 'defaultShouldLog'.
+--
+-- Since 1.4.10
+defaultShouldLogIO :: LogSource -> LogLevel -> IO Bool
+defaultShouldLogIO a b = return $ defaultShouldLog a b
+
 -- | Default implementation of 'yesodMiddleware'. Adds the response header
 -- \"Vary: Accept, Accept-Language\" and performs authorization checks.
 --
@@ -577,6 +615,16 @@ asyncHelper render scripts jscript jsLoc =
                     Nothing -> Nothing
                     Just j -> Just $ jelper j
 
+-- | Default formatting for log messages. When you use
+-- the template haskell logging functions for to log with information
+-- about the source location, that information will be appended to 
+-- the end of the log. When you use the non-TH logging functions, 
+-- like 'logDebugN', this function does not include source 
+-- information. This currently works by checking to see if the
+-- package name is the string \"\<unknown\>\". This is a hack,
+-- but it removes some of the visual clutter from non-TH logs.
+--
+-- Since 1.4.10
 formatLogMessage :: IO ZonedDate
                  -> Loc
                  -> LogSource
@@ -585,20 +633,24 @@ formatLogMessage :: IO ZonedDate
                  -> IO LogStr
 formatLogMessage getdate loc src level msg = do
     now <- getdate
-    return $
-        toLogStr now `mappend`
-        " [" `mappend`
-        (case level of
+    return $ mempty
+        `mappend` toLogStr now 
+        `mappend` " [" 
+        `mappend` (case level of
             LevelOther t -> toLogStr t
-            _ -> toLogStr $ drop 5 $ show level) `mappend`
-        (if T.null src
+            _ -> toLogStr $ drop 5 $ show level) 
+        `mappend` (if T.null src
             then mempty
-            else "#" `mappend` toLogStr src) `mappend`
-        "] " `mappend`
-        msg `mappend`
-        " @(" `mappend`
-        toLogStr (fileLocationToString loc) `mappend`
-        ")\n"
+            else "#" `mappend` toLogStr src) 
+        `mappend` "] " 
+        `mappend` msg 
+        `mappend` sourceSuffix
+        `mappend` "\n"
+    where 
+    sourceSuffix = if loc_package loc == "<unknown>" then "" else mempty
+        `mappend` " @(" 
+        `mappend` toLogStr (fileLocationToString loc) 
+        `mappend` ")"
 
 -- | Customize the cookies used by the session backend.  You may
 -- use this function on your definition of 'makeSessionBackend'.
