@@ -63,9 +63,14 @@ import Text.Blaze.Html.Renderer.Utf8 (renderHtmlBuilder)
 import Data.Conduit (Source, Flush (Chunk), ResumableSource, mapOutput)
 import Control.Monad.Trans.Resource (ResourceT)
 import Data.Conduit.Internal (ResumableSource (ResumableSource))
+import qualified Data.Conduit.Internal as CI
 
 import qualified Data.Aeson as J
+#if MIN_VERSION_aeson(0, 7, 0)
+import Data.Aeson.Encode (encodeToTextBuilder)
+#else
 import Data.Aeson.Encode (fromValue)
+#endif
 import qualified Blaze.ByteString.Builder.Char.Utf8 as Blaze
 import Data.Text.Lazy.Builder (toLazyText)
 import Yesod.Core.Types
@@ -115,6 +120,9 @@ instance ToContent Css where
 instance ToContent Javascript where
     toContent = toContent . toLazyText . unJavascript
 
+instance ToFlushBuilder builder => ToContent (CI.Pipe () () builder () (ResourceT IO) ()) where
+    toContent src = ContentSource $ CI.ConduitM (CI.mapOutput toFlushBuilder src >>=)
+
 instance ToFlushBuilder builder => ToContent (Source (ResourceT IO) builder) where
     toContent src = ContentSource $ mapOutput toFlushBuilder src
 instance ToFlushBuilder builder => ToContent (ResumableSource (ResourceT IO) builder) where
@@ -239,7 +247,11 @@ instance ToContent J.Value where
     toContent = flip ContentBuilder Nothing
               . Blaze.fromLazyText
               . toLazyText
+#if MIN_VERSION_aeson(0, 7, 0)
+              . encodeToTextBuilder
+#else
               . fromValue
+#endif
 instance HasContentType J.Value where
     getContentType _ = typeJson
 
