@@ -30,6 +30,7 @@ module Yesod.Core.Dispatch
     , defaultMiddlewaresNoLogging
       -- * WAI subsites
     , WaiSubsite (..)
+    , subHelper
     ) where
 
 import Prelude hiding (exp)
@@ -42,12 +43,12 @@ import qualified Network.Wai as W
 
 import Data.ByteString.Lazy.Char8 ()
 
-import Data.Text (Text, pack)
+import Data.Text (Text)
 import Data.Monoid (mappend)
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
 import qualified Blaze.ByteString.Builder
-import Network.HTTP.Types (status301)
+import Network.HTTP.Types (status301, status307)
 import Yesod.Routes.Parse
 import Yesod.Core.Types
 import Yesod.Core.Class.Yesod
@@ -68,6 +69,7 @@ import Control.Monad.Logger
 import Control.Monad (when)
 import qualified Paths_yesod_core
 import Data.Version (showVersion)
+import qualified System.Random.MWC as MWC
 
 -- | Convert the given argument into a WAI application, executable with any WAI
 -- handler. This function will provide no middlewares; if you want commonly
@@ -76,10 +78,12 @@ toWaiAppPlain :: YesodDispatch site => site -> IO W.Application
 toWaiAppPlain site = do
     logger <- makeLogger site
     sb <- makeSessionBackend site
+    gen <- MWC.createSystemRandom
     return $ toWaiAppYre $ YesodRunnerEnv
             { yreLogger = logger
             , yreSite = site
             , yreSessionBackend = sb
+            , yreGen = gen
             }
 
 toWaiAppYre :: YesodDispatch site => YesodRunnerEnv site -> W.Application
@@ -92,17 +96,18 @@ toWaiAppYre yre req =
   where
     site = yreSite yre
     sendRedirect :: Yesod master => master -> [Text] -> W.Application
-#if MIN_VERSION_wai(3, 0, 0)
     sendRedirect y segments' env sendResponse =
-         sendResponse $ W.responseLBS status301
-#else
-    sendRedirect y segments' env =
-         return $ W.responseLBS status301
-#endif
+         sendResponse $ W.responseLBS status
                 [ ("Content-Type", "text/plain")
                 , ("Location", Blaze.ByteString.Builder.toByteString dest')
                 ] "Redirecting"
       where
+        -- Ensure that non-GET requests get redirected correctly. See:
+        -- https://github.com/yesodweb/yesod/issues/951
+        status
+            | W.requestMethod env == "GET" = status301
+            | otherwise                    = status307
+
         dest = joinPath y (resolveApproot y env) segments' []
         dest' =
             if S.null (W.rawQueryString env)
@@ -130,10 +135,12 @@ toWaiApp site = do
 toWaiAppLogger :: YesodDispatch site => Logger -> site -> IO W.Application
 toWaiAppLogger logger site = do
     sb <- makeSessionBackend site
+    gen <- MWC.createSystemRandom
     let yre = YesodRunnerEnv
                 { yreLogger = logger
                 , yreSite = site
                 , yreSessionBackend = sb
+                , yreGen = gen
                 }
     messageLoggerSource
         site
@@ -157,19 +164,10 @@ toWaiAppLogger logger site = do
 warp :: YesodDispatch site => Int -> site -> IO ()
 warp port site = do
     logger <- makeLogger site
-    toWaiAppLogger logger site >>= Network.Wai.Handler.Warp.runSettings
-        Network.Wai.Handler.Warp.defaultSettings
-            { Network.Wai.Handler.Warp.settingsPort = port
-            {- FIXME
-            , Network.Wai.Handler.Warp.settingsServerName = S8.pack $ concat
-                [ "Warp/"
-                , Network.Wai.Handler.Warp.warpVersion
-                , " + Yesod/"
-                , showVersion Paths_yesod_core.version
-                , " (core)"
-                ]
-            -}
-            , Network.Wai.Handler.Warp.settingsOnException = const $ \e ->
+    toWaiAppLogger logger site >>= Network.Wai.Handler.Warp.runSettings (
+        Network.Wai.Handler.Warp.setPort port $
+        Network.Wai.Handler.Warp.setServerName serverValue $
+        Network.Wai.Handler.Warp.setOnException (\_ e ->
                 when (shouldLog' e) $
                 messageLoggerSource
                     site
@@ -177,15 +175,19 @@ warp port site = do
                     $(qLocation >>= liftLoc)
                     "yesod-core"
                     LevelError
-                    (toLogStr $ "Exception from Warp: " ++ show e)
-            }
+                    (toLogStr $ "Exception from Warp: " ++ show e)) $
+        Network.Wai.Handler.Warp.defaultSettings)
   where
-    shouldLog' =
-#if MIN_VERSION_warp(2,1,3)
-        Network.Wai.Handler.Warp.defaultShouldDisplayException
-#else
-        const True
-#endif
+    shouldLog' = Network.Wai.Handler.Warp.defaultShouldDisplayException
+
+serverValue :: S8.ByteString
+serverValue = S8.pack $ concat
+    [ "Warp/"
+    , Network.Wai.Handler.Warp.warpVersion
+    , " + Yesod/"
+    , showVersion Paths_yesod_core.version
+    , " (core)"
+    ]
 
 -- | A default set of middlewares.
 --
@@ -193,11 +195,7 @@ warp port site = do
 mkDefaultMiddlewares :: Logger -> IO W.Middleware
 mkDefaultMiddlewares logger = do
     logWare <- mkRequestLogger def
-#if MIN_VERSION_fast_logger(2, 0, 0)
         { destination = Network.Wai.Middleware.RequestLogger.Logger $ loggerSet logger
-#else
-        { destination = Logger logger
-#endif
         , outputFormat = Apache FromSocket
         }
     return $ logWare . defaultMiddlewaresNoLogging
