@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 module Yesod.Core.Internal.Util
     ( putTime
     , getTime
@@ -11,7 +12,12 @@ import           Data.Serialize (Get, Put, Serialize (..))
 import qualified Data.Text      as T
 import           Data.Time      (Day (ModifiedJulianDay, toModifiedJulianDay),
                                  DiffTime, UTCTime (..), formatTime)
+
+#if MIN_VERSION_time(1,5,0)
+import           Data.Time      (defaultTimeLocale)
+#else
 import           System.Locale  (defaultTimeLocale)
+#endif
 
 putTime :: UTCTime -> Put
 putTime (UTCTime d t) =
